/*
  whatis.cpp - Find the nickname associated with a hash.

  Jason Hood, 20 to 27 July, 2009.

  v1.01, 4 to 6, 11 August, 2009:
  * location has its own name in the database.
*/

#define PVERS "1.01"
#define PDATE "11 August, 2009"

#include "common.h"
#include "hashdb.h"

Common common;


int    fmt, flags;
HashDB hash;


int main( int argc, char* argv[] )
{
  if (argc == 1 ||
      ((argv[1][0] == '/' || argv[1][0] == '-') && argv[1][1] == '?') ||
      strcmp( argv[1], "--help" ) == 0)
  {
    puts( "WhatIs by Jason Hood <jadoxa@yahoo.com.au>.\n"
	  "Version " PVERS " (" PDATE ").  Freeware.\n"
	  "http://freelancer.adoxa.cjb.net/\n"
	  "\n"
	  "Translate hashes to nicknames (using CreateID's database).\n"
	  "\n"
	  "whatis [-dhmnNsSt] [dbfile] [-o[cim][n]] hash|nickname[|text]...\n"
	  "\n"
	  "-d\t\tuse DBFILE instead of \"hashes.dat\"\n"
	  "-h\t\tinclude the hex form of the hash\n"
	  "-m\t\tinclude the minus form of the hash\n"
	  "-n\t\ttreat numbers as nicknames, not hashes\n"
	  "-t\t\ttest for duplicated hashes\n"
	  "-s -S\t\tsearch for an entry containing TEXT\n"
	  "-o\t\toutput as:\n"
	  "  c\t\t  CSV\n"
	  "  i\t\t  IONCROSS\n"
	  "  m\t\t  MiniTrue\n"
	  "  n\t\treference nicknames rather than hashes for IONCROSS & MiniTrue\n"
	  "hash\t\tfind this hash number\n"
	  "nickname\tfind this nickname\n"
	  "\n"
	  "Nothing is output if the hash or nickname is not found.\n"
	  "-s will search the entire database; -S will stop at the first match.\n"
	  "The default search will use the names; -n will use nicknames and -N\n"
	  "will only use nicknames that have names.  If no text is provided\n"
	  "for -s, the entire database will be output."
	);
    return 0;
  }

  int	 nicknames = 0;
  LPCSTR dbname = "hashes.dat";
  int	 search = 0;
  bool	 test	= false;

  while (argv[1] && *argv[1] == '-' && !isdigit( argv[1][1] ))
  {
    bool fnd_d = false;
    while (*++argv[1])
    {
      switch (*argv[1])
      {
	case 'd':
	  if (!argv[2])
	  {
	    fputs( "Missing file name for -d.\n", stderr );
	    return 1;
	  }
	  dbname = argv[2];
	  fnd_d  = true;
	break;
	case 'h': flags |= outHex;   break;
	case 'm': flags |= outMinus; break;
	case 'n': nicknames = 1;     break;
	case 'N': nicknames = 2;     break;
	case 'o':
	  switch (argv[1][1])
	  {
	    case 'c': fmt = fmtCSV;      ++argv[1]; break;
	    case 'i': fmt = fmtIONCROSS; ++argv[1]; break;
	    case 'm': fmt = fmtMTR;      ++argv[1]; break;
	    default:
	      if (argv[1][1] != 'n')
	      {
		fprintf( stderr, "Unknown output format '%c'.\n", argv[1][1] );
		return 1;
	      }
	  }
	  if (argv[1][1] == 'n')
	  {
	    flags |= outName;
	    ++argv[1];
	  }
	break;
	case 's': search = 1;    break;
	case 'S': search = 2;    break;
	case 't': test   = true; break;
	default:
	  fprintf( stderr, "Unknown option '%c'.\n", *argv[1] );
	  return 1;
      }
    }
    ++argv;
    --argc;
    if (fnd_d)
    {
      ++argv;
      --argc;
    }
  }

  if (!hash.open( dbname, test ))
    return 1;

  if (search)
  {
    for (int i = 1; i < argc; ++i)
      _strlwr( argv[i] );
  }
  if (search == 1)
  {
    if (hash.search( --argc, ++argv, nicknames, true ))
    {
      do
      {
	output( hash.id(), hash.nick(), hash.name(), fmt, flags );
      } while (hash.search( argc, argv, nicknames, false ));
    }
  }
  else for (int i = 1; i < argc; ++i)
  {
    bool rc;
    if (search)
      rc = hash.search( 1, argv + i, nicknames, true );
    else
    {
      UINT id = 0;
      if (!nicknames)
      {
	char* end;
	id = (*argv[i] == '-') ? strtol(  argv[i], &end, 0 )
			       : strtoul( argv[i], &end, 0 );
	if (*end)
	  id = 0;
      }
      if (!id)
	id = get_hash( argv[i] );
      rc = hash.find( id );
    }
    if (rc)
      output( hash.id(), hash.nick(), hash.name(), fmt, flags );
  }

  hash.close();

  return 0;
}
